@@ -34,6 +34,8 @@
 #include <random>
 #include <ctime>
 #ifndef WIN_R_BUILD
+#include <thread>
+#include <chrono>
 #include <mutex>
 #include <condition_variable>
 #endif
@@ -143,13 +145,12 @@ protected:
   void computePredictionError();
   virtual void computePredictionErrorInternal() = 0;
 
-  void computeGiniImportance();
   void computePermutationImportance();
 
   // Multithreading methods for growing/prediction/importance, called by each thread
-  void growTreesInThread(uint thread_idx);
+  void growTreesInThread(uint thread_idx, std::vector<double>* variable_importance);
   void predictTreesInThread(uint thread_idx, const Data* prediction_data, bool oob_prediction);
-  void computeTreePermutationImportanceInThread(uint thread_idx);
+  void computeTreePermutationImportanceInThread(uint thread_idx, std::vector<double>* importance, std::vector<double>* variance);
 
   // Load forest from file
   void loadFromFile(std::string filename);
@@ -220,6 +221,10 @@ protected:
 
   // Computation progress (finished trees)
   size_t progress;
+#ifdef R_BUILD
+  size_t aborted_threads;
+  bool aborted;
+#endif
 
 private:
   DISALLOW_COPY_AND_ASSIGN(Forest);
